// ----------------------------------------------------------------------------
/// r91001_map.h
/// © 2009-2012 Sealevel Systems, Inc.
/// <summary>
/// R91001 Maps.
/// </summary>
///
/// <remarks>
/// This header declares several arrays useful for mapping R9-Development board
/// hardware to Linux driver entry points.
/// </remarks>
// ----------------------------------------------------------------------------

#ifndef __SEALEVEL_R91001_MAP_H__
#define __SEALEVEL_R91001_MAP_H__

#include "gpio.h"

#define ARRAY_LENGTH(a, b) (sizeof(a) / sizeof(b))

// Digital Output (Open-Collector Outputs) mapping data
int DIGITAL_OUTPUT_MAP[] = {
			ARM_GPIO_PB + 9,	// CPU Stat LED
			ARM_GPIO_PB + 8,	// OC1 - PWM1
			ARM_GPIO_PB + 27,	// OC2 - PWM2
			ARM_GPIO_PB + 29,	// OC3 - PWM3
			ARM_GPIO_PB + 24,	// OC4
			ARM_GPIO_PB + 25,	// OC5
			ARM_GPIO_PB + 26,	// OC6
			ARM_GPIO_PB + 28,	// OC7
			ARM_GPIO_PB + 30,	// OC8
		      };

int DIGITAL_OUTPUT_MAP_LENGTH = ARRAY_LENGTH(DIGITAL_OUTPUT_MAP, int);


// Digital Input (Opto Inputs) mapping data
int DIGITAL_INPUT_MAP[] = {
			-1,			// Place-keeper for index 0
			ARM_GPIO_PE + 0,	// OPTO1
			ARM_GPIO_PE + 1,	// OPTO2
			ARM_GPIO_PE + 2,	// OPTO3
			ARM_GPIO_PE + 3,	// OPTO4
			ARM_GPIO_PE + 4,	// OPTO5
			ARM_GPIO_PE + 5,	// OPTO6
			ARM_GPIO_PE + 6,	// OPTO7
			ARM_GPIO_PE + 7,	// OPTO8
		      };

int DIGITAL_INPUT_MAP_LENGTH = ARRAY_LENGTH(DIGITAL_INPUT_MAP, int);


// Analog Input mapping data
char *ANALOG_INPUT_DEV = "/dev/spidev0.0";
int ANALOG_INPUT_MUX[] = {
			ARM_GPIO_PB + 3,	// ADC Channel MUX Enable
			ARM_GPIO_PB + 0,	// ADC Channel MUX A0
			ARM_GPIO_PB + 1, 	// ADC Channel MUX A1
			ARM_GPIO_PB + 2, 	// ADC Channel MUX A2
		      };
int ANALOG_INPUT_MUX_BITS = 3;

#define ZERO_TO_FIVE		0x88
#define MINUS_FIVE_TO_FIVE	0x80
#define ZERO_TO_TEN		0x8C
#define MINUS_TEN_TO_TEN	0x84


// Quadrature Counter mapping data
char *COUNTER_DEV[] = {
			"/dev/spidev0.1",
			"/dev/spidev0.2",
		      };
int COUNTER_DEV_LENGTH = 2;

int COUNTER_FLAG[] = {
			ARM_GPIO_PC + 27,
			ARM_GPIO_PC + 28,
		     };
int COUNTER_FLAG_LENGTH = ARRAY_LENGTH(DIGITAL_INPUT_MAP, int);


#endif
